@@ -187,10 +187,12 @@ int FASTDB_DLL_ENTRY cli_open(char const* server_url,
                               int         reconnect_timeout_sec);
 
 enum cli_open_attributes { 
-    cli_open_default    = 0x0, 
-    cli_open_readonly   = 0x1, 
-    cli_open_truncate   = 0x2,
-    cli_open_concurrent = 0x4
+    cli_open_default    = 0x00, 
+    cli_open_readonly   = 0x01, 
+    cli_open_truncate   = 0x02,
+    cli_open_concurrent = 0x04,
+    cli_open_force_read = 0x08,
+    cli_open_no_sync    = 0x10
 };
 /*********************************************************************
  * cli_create
@@ -242,8 +244,8 @@ int FASTDB_DLL_ENTRY cli_create(char_t const* databaseName,
 int FASTDB_DLL_ENTRY cli_create_replication_node(int         nodeId,
                                                  int         nServers,
                                                  char*       nodeNames[],
-                                                 char const* databaseName, 
-                                                 char const* filePath, 
+                                                 char_t const* databaseName, 
+                                                 char_t const* filePath, 
                                                  int         openAttr, 
                                                  size_t      initDatabaseSize,
                                                  size_t      extensionQuantum,
@@ -998,6 +1000,19 @@ int FASTDB_DLL_ENTRY cli_backup(int session, char_t const* file_name, int compac
 int FASTDB_DLL_ENTRY cli_schedule_backup(int session, char_t const* file_name, int period);
 
 
+/**
+ * cli_get_wrapping_rectangle
+ *     Get wrapping rectangle for all objects in table
+ * Parameters:
+ *     session - session descriptor returned by cli_open
+ *     table - table name
+ *     field - sptial index key field name
+ *     rect - pointer to wrapper rectangle
+ * Returns:
+ *     result code as described in cli_result_code enum
+ */
+int FASTDB_DLL_ENTRY cli_get_wrapping_rectangle(int session, char const* table, char const* field, cli_rectangle_t* rect);
+
 
 #ifdef __cplusplus
 }
