@@ -15,21 +15,21 @@ BEGIN_FASTDB_NAMESPACE
 
 class dbOrderByNode;
 
-class FASTDB_DLL_ENTRY dbSelection { 
+class FASTDB_DLL_ENTRY dbSelection {
   public:
     enum { quantum = 1024 };
-    class segment { 
+    class segment {
       public:
         segment* prev;
         segment* next;
         int      nRows;
         oid_t    rows[quantum];
 
-        segment(segment* after) { 
+        segment(segment* after) {
             prev = after;
             next = NULL;
             nRows = 0;
-        }       
+        }
     };
     segment*  first;
     segment*  last;
@@ -40,22 +40,22 @@ class FASTDB_DLL_ENTRY dbSelection {
     segment*  createNewSegment(segment* after);
 
     void add(oid_t oid) {
-        if (last == NULL) { 
+        if (last == NULL) {
             first = last = createNewSegment(NULL);
-        } else if (last->nRows == quantum) { 
+        } else if (last->nRows == quantum) {
             last = last->next = createNewSegment(last);
         }
         last->rows[last->nRows++] = oid;
         nRows += 1;
     }
-   
+
     void sort(dbDatabase* db, dbOrderByNode* order);
     static int compare(oid_t a, oid_t b, dbOrderByNode* order);
 
     void toArray(oid_t* oids) const;
     void truncate(cardinality_t from, cardinality_t length);
 
-    dbSelection() { 
+    dbSelection() {
         nRows = 0;
         pos = 0;
         first = curr = last = NULL;
@@ -64,7 +64,7 @@ class FASTDB_DLL_ENTRY dbSelection {
     void reset();
 };
 
-enum dbCursorType { 
+enum dbCursorType {
     dbCursorViewOnly,
     dbCursorForUpdate
 };
@@ -72,12 +72,13 @@ enum dbCursorType {
 /**
  * Base class for all cursors
  */
-class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List { 
+class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
     friend class dbAnyContainer;
     friend class dbDatabase;
     friend class dbHashTable;
     friend class dbTtreeNode;
     friend class dbRtreePage;
+    friend class dbRtree;
     friend class dbSubSql;
     friend class dbStatement;
     friend class dbServer;
@@ -94,7 +95,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Remove current record
      */
     void remove();
-    
+
     /**
      * Checks whether selection is empty
      * @return true if there is no current record
@@ -105,7 +106,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Check whether this cursor can be used for update
      * @return true if it is update cursor
      */
-    bool isUpdateCursor() const { 
+    bool isUpdateCursor() const {
         return type == dbCursorForUpdate;
     }
 
@@ -122,14 +123,14 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      *  If <code>arr</code> is null, then new array is created by  new oid_t[] and returned by this method
      * @return if <code>arr</code> is not null, then <code>arr</code>, otherwise array created by this method
      */
-    oid_t* toArrayOfOid(oid_t* arr) const; 
+    oid_t* toArrayOfOid(oid_t* arr) const;
 
     /**
      * Execute query.
      * @param query selection criteria
      * @param aType cursor type: <code>dbCursorForUpdate, dbCursorViewOnly</code>
      * @param paramStruct pointer to structure with parameters. If you want to create reentrant precompiled query, i.e.
-     * query which can be used concurrently by different threadsm you should avoid to use static variables in 
+     * query which can be used concurrently by different threadsm you should avoid to use static variables in
      * such query, and instead of it place paramters into some structure, specify in query relative offsets to the parameters,
      * fill local structure and pass pointer to it to select method.
      * @return number of selected records
@@ -140,22 +141,22 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
         paramBase = paramStruct;
         db->select(this, query);
         paramBase = NULL;
-        if (gotoFirst() && prefetch) { 
+        if (gotoFirst() && prefetch) {
             fetch();
         }
         return selection.nRows;
-    } 
-    
+    }
+
     /**
      * Execute query with default cursor type.
      * @param query selection criteria
      * @param paramStruct pointer to structure with parameters.
      * @return number of selected records
-     */    
-    cardinality_t select(dbQuery& query, void* paramStruct = NULL) { 
+     */
+    cardinality_t select(dbQuery& query, void* paramStruct = NULL) {
         return select(query, defaultType, paramStruct);
     }
-     
+
     /**
      * Execute query.
      * @param condition selection criteria
@@ -163,18 +164,18 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * @param paramStruct pointer to structure with parameters.
      * @return number of selected records
      */
-    cardinality_t select(char const* condition, dbCursorType aType, void* paramStruct = NULL) { 
+    cardinality_t select(char const* condition, dbCursorType aType, void* paramStruct = NULL) {
         dbQuery query(condition);
         return select(query, aType, paramStruct);
-    } 
+    }
 
     /**
      * Execute query with default cursor type.
      * @param condition selection criteria
      * @param paramStruct pointer to structure with parameters.
      * @return number of selected records
-     */    
-    cardinality_t select(char const* condition, void* paramStruct = NULL) { 
+     */
+    cardinality_t select(char const* condition, void* paramStruct = NULL) {
         return select(condition, defaultType, paramStruct);
     }
 
@@ -182,64 +183,64 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Select all records from the table
      * @param aType cursor type: <code>dbCursorForUpdate, dbCursorViewOnly</code>
      * @return number of selected records
-     */    
-    cardinality_t select(dbCursorType aType) { 
+     */
+    cardinality_t select(dbCursorType aType) {
         type = aType;
         reset();
-        db->select(this); 
-        if (gotoFirst() && prefetch) { 
+        db->select(this);
+        if (gotoFirst() && prefetch) {
             fetch();
         }
         return selection.nRows;
-    } 
+    }
 
     /**
      * Select all records from the table with default cursor type
      * @return number of selected records
-     */    
+     */
     cardinality_t select() {
         return select(defaultType);
     }
 
     /**
-     * Select all records from the table with specfied value of the key
+     * Select all records from the table with specified value of the key
      * @param key name of the key field
      * @param value searched value of the key
      * @return number of selected records
-     */    
+     */
     cardinality_t selectByKey(char const* key, void const* value);
 
     /**
-     * Select all records from the table with specfied value of the key
+     * Select all records from the table with specified value of the key
      * @param field key field
      * @param value searched value of the key
      * @return number of selected records
-     */    
+     */
     cardinality_t selectByKey(dbFieldDescriptor* field, void const* value);
 
     /**
-     * Select all records from the table with specfied range of the key values
+     * Select all records from the table with specified range of the key values
      * @param key name of the key field
      * @param minValue inclusive low bound for key values, if <code>NULL</code> then there is no low bound
      * @param maxValue inclusive high bound for key values, if <code>NULL</code> then there is no high bound
      * @return number of selected records
-     */    
+     */
     cardinality_t selectByKeyRange(char const* key, void const* minValue, void const* maxValue);
 
     /**
-     * Select all records from the table with specfied range of the key values
+     * Select all records from the table with specified range of the key values
      * @param field key field
      * @param minValue inclusive low bound for key values, if <code>NULL</code> then there is no low bound
      * @param maxValue inclusive high bound for key values, if <code>NULL</code> then there is no high bound
      * @return number of selected records
-     */    
+     */
     cardinality_t selectByKeyRange(dbFieldDescriptor* field, void const* minValue, void const* maxValue);
 
     /**
      * Update current record. You should changed value of current record before and then call
      * update method to save changes to the database
      */
-    void update() { 
+    void update() {
         if (type != dbCursorForUpdate) {
             db->handleError(dbDatabase::ReadonlyCursor);
         }
@@ -270,7 +271,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Specify maximal number of records to be selected
      */
     void setSelectionLimit(cardinality_t lim) { limit = lim; }
-    
+
     /**
      * Remove selection limit
      */
@@ -292,7 +293,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
     void enableCheckForDuplicates(bool enabled) {
         checkForDuplicatedIsEnabled = enabled;
     }
-        
+
 
     /**
      * Reset cursor
@@ -303,19 +304,19 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Check whether current record is the last one in the selection
      * @return true if next() method will return <code>NULL</code>
      */
-    bool isLast() const; 
+    bool isLast() const;
 
     /**
      * Check whether current record is the first one in the selection
      * @return true if prev() method will return <code>NULL</code>
      */
-    bool isFirst() const; 
+    bool isFirst() const;
 
     /**
      * Freeze cursor. This method makes it possible to save current state of cursor, close transaction to allow
-     * other threads to proceed, and then later restore state of the cursor using unfreeze method and continue 
+     * other threads to proceed, and then later restore state of the cursor using unfreeze method and continue
      * traversal through selected records.
-     */     
+     */
     void freeze();
 
     /**
@@ -325,7 +326,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
 
     /**
      * Skip specified number of records
-     * @param n if positive then skip <code>n</code> records forward, if negative then skip <code>-n</code> 
+     * @param n if positive then skip <code>n</code> records forward, if negative then skip <code>-n</code>
      * records backward
      * @return <code>true</code> if specified number of records was successfully skipped, <code>false</code> if
      * there is no next (<code>n &gt; 0</code>) or previous (<code>n &lt; 0</code>) record in the selction.
@@ -350,7 +351,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Set table for the cursor
      * @param aTable table which records will be iterated
      */
-    void setTable(dbTableDescriptor* aTable) { 
+    void setTable(dbTableDescriptor* aTable) {
         table = aTable;
         db = aTable->db;
     }
@@ -359,7 +360,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Set destination for selected record
      * rec - buffer to which fields of current record will be fetched
      */
-    void setRecord(void* rec) { 
+    void setRecord(void* rec) {
         record = (byte*)rec;
     }
 
@@ -367,7 +368,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Get pointer to the location where fields of the current record are fetched
      * @return pointer to the memory location set by cursor constructor or setRecord method
      */
-    void* getRecord() { 
+    void* getRecord() {
         return record;
     }
 
@@ -380,13 +381,13 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
 
     /**
      * Fetch current record.
-     * You should use this method only if prefetch mode is disabled 
+     * You should use this method only if prefetch mode is disabled
      */
-    void fetch() { 
+    void fetch() {
         if (db->currIndex[currId] & (dbInternalObjectMarker|dbFreeHandleMarker)) {
             db->handleError(dbDatabase::AccessToDeletedObject);
         }
-        table->columns->fetchRecordFields(record, 
+        table->columns->fetchRecordFields(record,
                                           (byte*)db->getRow(currId));
     }
 
@@ -394,8 +395,8 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Check if there is more records in the selection
      */
     bool hasNext() const;
-    
-  protected: 
+
+  protected:
     dbDatabase*        db;
     dbTableDescriptor* table;
     dbCursorType       type;
@@ -421,44 +422,44 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
     cardinality_t      nSkipped;
 
     void*              paramBase;
-    
+
     void checkForDuplicates();
     void deallocateBitmap();
 
-    bool isMarked(oid_t oid) { 
+    bool isMarked(oid_t oid) {
         return bitmap != NULL && (bitmap[oid >> 5] & (1 << (oid & 31))) != 0;
     }
 
-    void setStatementLimit(dbQuery const& q) { 
+    void setStatementLimit(dbQuery const& q) {
         stmtLimitStart = q.stmtLimitStartPtr != NULL ? (nat4)*q.stmtLimitStartPtr : q.stmtLimitStart;
         stmtLimitLen = q.stmtLimitLenPtr != NULL ? (nat4)*q.stmtLimitLenPtr : q.stmtLimitLen;
     }
 
-    void truncateSelection() { 
+    void truncateSelection() {
         selection.truncate(stmtLimitStart, stmtLimitLen);
     }
 
-    void mark(oid_t oid) { 
-        if (bitmap != NULL) { 
+    void mark(oid_t oid) {
+        if (bitmap != NULL) {
             bitmap[oid >> 5] |= 1 << (oid & 31);
         }
-    }    
+    }
 
-    bool add(oid_t oid) { 
-        if (selection.nRows < limit && selection.nRows < stmtLimitLen) { 
-            if (nSkipped < stmtLimitStart) { 
+    bool add(oid_t oid) {
+        if (selection.nRows < limit && selection.nRows < stmtLimitLen) {
+            if (nSkipped < stmtLimitStart) {
                 nSkipped += 1;
                 return true;
             }
-            if (eliminateDuplicates) { 
-                if (bitmap[oid >> 5] & (1 << (oid & 31))) { 
+            if (eliminateDuplicates) {
+                if (bitmap[oid >> 5] & (1 << (oid & 31))) {
                     return true;
                 }
                 bitmap[oid >> 5] |= 1 << (oid & 31);
-            } 
+            }
             selection.add(oid);
             return selection.nRows < limit;
-        } 
+        }
         return false;
     }
 
@@ -466,18 +467,18 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
     byte* fetchPrev();
 
     bool gotoNext();
-    bool gotoPrev(); 
+    bool gotoPrev();
     bool gotoFirst();
     bool gotoLast();
 
-    
+
     bool moveNext();
     bool movePrev();
 
     void setCurrent(dbAnyReference const& ref);
 
-    void adjustReferences(size_t base, size_t size, size_t shift) { 
-        if (currId != 0 && record != NULL) { 
+    void adjustReferences(size_t base, size_t size, size_t shift) {
+        if (currId != 0 && record != NULL) {
             table->columns->adjustReferences(record, base, size, shift);
         }
     }
@@ -490,7 +491,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
         limit = dbDefaultSelectionLimit;
         prefetch = rec != NULL;
         removed = false;
-        bitmap = NULL; 
+        bitmap = NULL;
         bitmapSize = 0;
         eliminateDuplicates = false;
         checkForDuplicatedIsEnabled = true;
@@ -500,7 +501,7 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
         nSkipped = 0;
     }
   public:
-    dbAnyCursor() 
+    dbAnyCursor()
     : table(NULL),type(dbCursorViewOnly),defaultType(dbCursorViewOnly),
           allRecords(false),currId(0),record(NULL)
     {
@@ -525,28 +526,28 @@ class FASTDB_DLL_ENTRY dbAnyCursor : public dbL2List {
  * Cursor template parameterized by table class
  */
 template<class T>
-class dbCursor : public dbAnyCursor { 
+class dbCursor : public dbAnyCursor {
   private:
     // It is not possible to copy cursors
-    dbCursor<T> operator = (dbCursor<T> const& src) { 
+    dbCursor<T> operator = (dbCursor<T> const& src) {
         return *this;
-    } 
+    }
     dbCursor(dbCursor<T> const& src) {assert(0);}
 
   protected:
     T record;
-    
+
   public:
     /**
      * Cursor constructor
      * @param type cursor type (dbCursorViewOnly by default)
      */
-    dbCursor(dbCursorType type = dbCursorViewOnly) 
+    dbCursor(dbCursorType type = dbCursorViewOnly)
         : dbAnyCursor(T::dbDescriptor, type, (byte*)&record) {}
 
     /**
      * Cursor constructor with explicit specification of database.
-     * This cursor should be used for unassigned tables. 
+     * This cursor should be used for unassigned tables.
      * @param aDb database in which table lookup is performed
      * @param type cursor type (dbCursorViewOnly by default)
      */
@@ -555,7 +556,7 @@ class dbCursor : public dbAnyCursor {
     {
         db = aDb;
         dbTableDescriptor* theTable = db->lookupTable(table);
-        if (theTable != NULL) { 
+        if (theTable != NULL) {
             table = theTable;
         }
     }
@@ -564,14 +565,14 @@ class dbCursor : public dbAnyCursor {
      * Get pointer to the current record
      * @return pointer to the current record or <code>NULL</code> if there is no current record
      */
-    T* get() { 
-        return currId == 0 ? (T*)NULL : &record; 
+    T* get() {
+        return currId == 0 ? (T*)NULL : &record;
     }
-    
+
     /**
      * Get next record
      * @return pointer to the next record or <code>NULL</code> if there is no next record
-     */     
+     */
     T* next() {
         return (T*)fetchNext();
     }
@@ -579,8 +580,8 @@ class dbCursor : public dbAnyCursor {
     /**
      * Get previous record
      * @return pointer to the previous record or <code>NULL</code> if there is no previous record
-     */     
-    T* prev() { 
+     */
+    T* prev() {
         return (T*)fetchPrev();
     }
 
@@ -588,7 +589,7 @@ class dbCursor : public dbAnyCursor {
      * Get pointer to the first record
      * @return pointer to the first record or <code>NULL</code> if no records were selected
      */
-    T* first() { 
+    T* first() {
         if (gotoFirst()) {
             fetch();
             return &record;
@@ -600,20 +601,20 @@ class dbCursor : public dbAnyCursor {
      * Get pointer to the last record
      * @return pointer to the last record or <code>NULL</code> if no records were selected
      */
-    T* last() { 
+    T* last() {
         if (gotoLast()) {
             fetch();
             return &record;
         }
         return NULL;
-    }    
-    
+    }
+
     /**
      * Position cursor on the record with the specified OID
      * @param ref reference to the object
      * @return poistion of the record in the selection or -1 if record with such OID is not in selection
      */
-    int seek(dbReference<T> const& ref) { 
+    int seek(dbReference<T> const& ref) {
         return dbAnyCursor::seek(ref.getOid());
     }
 
@@ -621,7 +622,7 @@ class dbCursor : public dbAnyCursor {
      * Overloaded operator for accessing components of the current record
      * @return pointer to the current record
      */
-    T* operator ->() { 
+    T* operator ->() {
         if (currId == 0) {
             db->handleError(dbDatabase::NoCurrentRecord);
         }
@@ -633,16 +634,16 @@ class dbCursor : public dbAnyCursor {
      * @param ref reference to the record
      * @return pointer to the referenced record
      */
-    T* at(dbReference<T> const& ref) { 
+    T* at(dbReference<T> const& ref) {
         setCurrent(ref);
         return &record;
     }
-    
+
     /**
      * Convert selection to array of reference
      * @param arr [OUT] array of refeences in which references to selected recrods will be placed
      */
-    void toArray(dbArray< dbReference<T> >& arr) const { 
+    void toArray(dbArray< dbReference<T> >& arr) const {
         arr.resize(selection.nRows);
         toArrayOfOid((oid_t*)arr.base());
     }
@@ -651,7 +652,7 @@ class dbCursor : public dbAnyCursor {
      * Get current object idenitifer
      * @return reference to the current record
      */
-    dbReference<T> currentId() const { 
+    dbReference<T> currentId() const {
         return dbReference<T>(currId);
     }
 
@@ -664,7 +665,7 @@ class dbCursor : public dbAnyCursor {
     }
 };
 
-class dbParallelQueryContext { 
+class dbParallelQueryContext {
   public:
     dbDatabase* const      db;
     dbCompiledQuery* const query;
@@ -673,9 +674,9 @@ class dbParallelQueryContext {
     dbAnyCursor*           cursor;
     dbSelection            selection[dbMaxParallelSearchThreads];
 
-    void search(int i); 
+    void search(int i);
 
-    dbParallelQueryContext(dbDatabase* aDb, dbTable* aTable, 
+    dbParallelQueryContext(dbDatabase* aDb, dbTable* aTable,
                            dbCompiledQuery* aQuery, dbAnyCursor* aCursor)
       : db(aDb), query(aQuery), firstRow(aTable->firstRow), table(aTable), cursor(aCursor) {}
 };
