@@ -91,7 +91,7 @@ void dbSelection::truncate(cardinality_t from, cardinality_t length)
             if (n > length) { 
                 n = length;
             }
-            if (dst->nRows == pos) { 
+            if (size_t(dst->nRows) == pos) { 
                 dst = dst->next;
                 pos = 0;
             }
@@ -102,7 +102,7 @@ void dbSelection::truncate(cardinality_t from, cardinality_t length)
             pos += n;
             length -= n;
             nRows += n;
-            if ((from += n) == src->nRows) { 
+            if ((from += n) == cardinality_t(src->nRows)) { 
                 if ((src = src->next) == NULL) { 
                     break;
                 }
@@ -247,10 +247,12 @@ int dbSelection::compare(oid_t o1, oid_t o2, dbOrderByNode* order)
                 }
                 break;
               case dbField::tpArray:
-                diff = order->field->_comparator(&dbArray<char>(p + ((dbVarying*)(p + offs))->offs, ((dbVarying*)(p + offs))->size), 
-                                                 &dbArray<char>(q + ((dbVarying*)(q + offs))->offs, ((dbVarying*)(q + offs))->size),
-                                                 0);
-                break;
+              {
+                  dbArray<char> a1(p + ((dbVarying*)(p + offs))->offs, ((dbVarying*)(p + offs))->size);
+                  dbArray<char> a2(q + ((dbVarying*)(q + offs))->offs, ((dbVarying*)(q + offs))->size);                  
+                  diff = order->field->_comparator(&a1, &a2, 0);
+                  break;
+              }
               case dbField::tpRawBinary:
                 diff = order->field->_comparator(p + offs, q + offs, order->field->dbsSize);
                 break;
@@ -269,7 +271,6 @@ int dbSelection::compare(oid_t o1, oid_t o2, dbOrderByNode* order)
 #ifdef USE_HEAP_SORT
 
 #define ELEM(i)   index[(i-1)/quantum]->rows[(i-1)%quantum]
-#define ROW(i)    db->getRow(ELEM(i))
 #define SWAP(i,j) temp = ELEM(i), ELEM(i) = ELEM(j), ELEM(j) = temp
 
 void dbSelection::sort(dbDatabase* db, dbOrderByNode* order)
@@ -287,45 +288,43 @@ void dbSelection::sort(dbDatabase* db, dbOrderByNode* order)
     }
     for (i = n/2, j = i; i >= 1; i--) { 
         k = i;
-        oid_t topId = ELEM(k);
-        dbRecord* top = db->getRow(topId);
+        oid_t top = ELEM(k);
         do { 
-            if (k*2 == n || compare(ROW(k*2), ROW(k*2+1), order) > 0) { 
-                if (compare(top, ROW(k*2), order) >= 0) {
+            if (k*2 == n || compare(ELEM(k*2), ELEM(k*2+1), order) > 0) { 
+                if (compare(top, ELEM(k*2), order) >= 0) {
                     break;
                 }
                 ELEM(k) = ELEM(k*2);
                 k = k*2;
             } else { 
-                if (compare(top, ROW(k*2+1), order) >= 0) {
+                if (compare(top, ELEM(k*2+1), order) >= 0) {
                     break;
                 }
                 ELEM(k) = ELEM(k*2+1);
                 k = k*2+1;
             }
         } while (k <= j);
-        ELEM(k) = topId; 
+        ELEM(k) = top; 
     }
     for (i = n; i >= 2; i--) { 
         SWAP(1, i);
-        oid_t topId = ELEM(1);
-        dbRecord* top = db->getRow(topId);
+        oid_t top = ELEM(1);
         for (k = 1, j = (i-1)/2; k <= j;) { 
-            if (k*2 == i-1 || compare(ROW(k*2), ROW(k*2+1), order) > 0) { 
-                if (compare(top, ROW(k*2), order) >= 0) {
+            if (k*2 == i-1 || compare(ELEM(k*2), ELEM(k*2+1), order) > 0) { 
+                if (compare(top, ELEM(k*2), order) >= 0) {
                     break;
                 }
                 ELEM(k) = ELEM(k*2);
                 k = k*2;
             } else { 
-                if (compare(top, ROW(k*2+1), order) >= 0) {
+                if (compare(top, ELEM(k*2+1), order) >= 0) {
                     break;
                 }
                 ELEM(k) = ELEM(k*2+1);
                 k = k*2+1;
             }
         } 
-        ELEM(k) = topId; 
+        ELEM(k) = top; 
     }
     delete[] index;
 }
