@@ -573,9 +573,9 @@ void dbFile::unprotect(size_t pos, size_t size)
 inline bool SetFilePosition(HANDLE fh, size_t pos) 
 {
 #ifdef _WIN64
-    DWORD posHigh = (DWORD)(pos >> 32);
+    LONG posHigh = (LONG)(pos >> 32);
     DWORD posLow = SetFilePointer(fh, (LONG)pos, &posHigh, FILE_BEGIN);
-    retrun ((size_t(posHigh) << 32) | posLow) == pos;
+    return ((size_t(posHigh) << 32) | posLow) == pos;
 #else 
     return SetFilePointer(fh, pos, NULL, FILE_BEGIN) == pos; 
 #endif
@@ -696,7 +696,9 @@ int dbFile::open(char_t const* fileName, char_t const* sharedName, int flags,
         return status;
     } 
     if (flags & force_read) {
-        forceRead(mmapAddr, fileSize);
+        if (forceRead(mmapAddr, fileSize) != 0) { 
+			this->flags &= ~force_read; // this hack is needed to prevent compilter from elimination of this function call
+		}
     }
     if (status != ERROR_ALREADY_EXISTS && mmapSize > fileSize)
         //      && osinfo.dwPlatformId != VER_PLATFORM_WIN32_NT) 
@@ -721,8 +723,8 @@ int dbFile::open(char_t const* fileName, char_t const* sharedName, int flags,
     currUpdateCount = new int[nPages];
 
     if (replicationSupport) { 
-        char* cFileName = new char[strlen(fileName) + 5];
-        strcat(strcpy(cFileName, fileName), ".cnt");
+        char_t* cFileName = new char_t[_tcslen(fileName) + 5];
+        _tcscat(_tcscpy(cFileName, fileName), _T(".cnt"));
         
 #ifdef DISKLESS_CONFIGURATION
         cfh = INVALID_HANDLE_VALUE;
@@ -1277,10 +1279,10 @@ int dbFile::open(char const* name, char const*, int flags, size_t initSize, bool
         int open_flags = ((flags & read_only) ? O_RDONLY : O_RDWR/*|O_DSYNC*/|O_CREAT) 
             | ((flags & truncate) ? O_TRUNC : 0)
             | ((flags & (no_buffering|write_through)) ? O_DIRECT : 0);
+        struct stat statBuf;
 #ifdef VXWORKS
         // Check if the file exists
         bool bFileExists = true;
-        struct stat statBuf;
         if (::stat(name, &statBuf) != OK)
         {
             if (errno == ENOENT || errno == ENOTDIR || errno == ENAMETOOLONG || errno == ENODEV) {
@@ -1312,16 +1314,20 @@ int dbFile::open(char const* name, char const*, int flags, size_t initSize, bool
             directio(fd, DIRECTIO_ON);
         }
 #endif
-        mmapSize = lseek(fd, 0, SEEK_END); 
-        if ((flags & read_only) == 0 && mmapSize == 0) { 
-            mmapSize = initSize;
-            if (ftruncate(fd, mmapSize) != ok) {
-                status = errno;
-                if (fd >= 0) { 
-                    ::close(fd);
+        if (::fstat(fd, &statBuf) != 0 || (statBuf.st_mode & S_IFREG)) {         
+            mmapSize = lseek(fd, 0, SEEK_END); 
+            if ((flags & read_only) == 0 && mmapSize == 0) { 
+                mmapSize = initSize;
+                if (ftruncate(fd, mmapSize) != ok) {
+                    status = errno;
+                    if (fd >= 0) { 
+                        ::close(fd);
+                    }
+                    return status;
                 }
-                return status;
             }
+        } else { 
+            mmapSize = initSize;
         }
     }
 #endif // DISKLESS_CONFIGURATION
@@ -1378,7 +1384,9 @@ int dbFile::open(char const* name, char const*, int flags, size_t initSize, bool
         return status;
     }
     if (flags & force_read) {
-        forceRead(mmapAddr, mmapSize);
+        if (forceRead(mmapAddr, mmapSize) != 0) { 
+			this->flags &= ~force_read; // this hack is needed to prevent compilter from elimination of this function call
+		}
     }
 #endif // NO_MMAP
 #endif // USE_SYSV_SHARED_MEMORY && DISKLESS_CONIFIGURATION
@@ -1552,7 +1560,7 @@ int dbFile::create(const char* name, int flags)
     assert(!(flags & ram_file));
     this->flags = flags;
     mmapAddr = NULL;
-    fd = ::open(name, O_RDWR|O_TRUNC|O_CREAT|((flags & no_buffering) ? O_DIRECT : 0), 0666);
+    fd = ::open(name, O_RDWR|((flags & truncate) ? O_TRUNC : 0)|O_CREAT|((flags & no_buffering) ? O_DIRECT : 0), 0666);
     if (fd < 0) { 
         return errno;
     }
