@@ -402,7 +402,7 @@ static void exportRecord(dbFieldDescriptor* fieldList, FILE* out, byte* src, int
                 }
                 fprintf(out, "/><vertex");
                 for (i = 0; i < rectangle::dim; i++) { 
-                    fprintf(out, " c%d=\"%d\")", i, r.boundary[rectangle::dim+i]);
+                    fprintf(out, " c%d=\"%d\"", i, r.boundary[rectangle::dim+i]);
                 }
                 fprintf(out, "/></rectangle>");
             }
@@ -669,6 +669,7 @@ dbXmlScanner::token dbXmlScanner::scan()
             if (!isdigit(ch) && ch != '-' && ch != '+' && ch != '.' && ch != 'E') { 
                 unget(ch);
                 sconst[i] = '\0';
+                slen = i;
                 if (floatingPoint) { 
                    return sscanf(sconst, "%lf%n", &fconst, &pos) == 1 && pos == i
                        ? xml_fconst : xml_error;
@@ -812,7 +813,8 @@ bool dbDatabase::importField(char* terminator, dbFieldDescriptor* fd, byte* rec,
         *(oid_t*)dst = mapId(id);
         break;
       case dbField::tpRectangle:
-        if (!EXPECT("rectangle")
+        if (!EXPECT(dbXmlScanner::xml_lt)
+            || !EXPECT("rectangle")
             || !EXPECT(dbXmlScanner::xml_gt)
             || !EXPECT(dbXmlScanner::xml_lt)
             || !EXPECT("vertex"))
@@ -994,7 +996,7 @@ bool dbDatabase::importField(char* terminator, dbFieldDescriptor* fd, byte* rec,
               break;            
           }
 #if STLPORT_WITH_VC6
-          ((CHAR_STRING*)dst)->assign(scanner.getString(), scanner.getStringLength());
+          ((std::string*)dst)->assign(scanner.getString(), scanner.getStringLength());
 #else
           new (dst) tmp_basic_string(scanner.getString(), scanner.getStringLength(), xmlContext->stdStringAlloc);
 #endif
